@@ -22,14 +22,13 @@ class TaskQueueDelegate(object):
 
 class Task(Primitive):
 
-    def __init__(self, name=None, after=None):
+    def __init__(self, name=None):
         Primitive.__init__(self, name=name)
         self.Created = time.time()
         self.Queued = None
         self.Started = None
         self.Ended = None
         self._Promise = None
-        self.After = after
     
     #def __call__(self):
     #    pass
@@ -72,7 +71,7 @@ class FunctionTask(Task):
         result = self.F(*self.Params, **self.Args)
         self.F = self.Params = self.Args = None
         return result
-
+        
 class TaskQueue(Primitive):
     
     class ExecutorThread(PyThread):
@@ -90,17 +89,17 @@ class TaskQueue(Primitive):
                 else:
                     result = task.run()
                 task.end()
-                self.Queue.taskEnded(self.Task, result)
                 promise = task._Promise
                 if promise is not None:
                     promise.complete(result)
+                self.Queue.taskEnded(self.Task, result)
             except:
                 task.end()
                 exc_type, value, tb = sys.exc_info()
-                self.Queue.taskFailed(self.Task, exc_type, value, tb)
                 promise = task._Promise
                 if promise is not None:
                     promise.exception(exc_type, value, tb)
+                self.Queue.taskFailed(self.Task, exc_type, value, tb)
             finally:
                 self.Queue.threadEnded(self)
                 self.Queue = None
@@ -119,10 +118,8 @@ class TaskQueue(Primitive):
         for t in tasks:
             self.addTask(t)
 
-    @synchronized
-    def addTask(self, task, timeout = None, after=None, promise_data=None):
+    def addTask(self, task, timeout = None, promise_data=None):
         #print "addTask() entry"
-        task.After = task.After or after
         self.Queue.append(task, timeout=timeout)
         #print("queue.append done", self.counts())
         task.enqueue()
@@ -130,7 +127,7 @@ class TaskQueue(Primitive):
         self.startThreads()
         return task._Promise
         
-    add = addTask
+    append = add = addTask
         
     def __iadd__(self, task):
         return self.addTask(task)
@@ -138,13 +135,7 @@ class TaskQueue(Primitive):
     def __lshift__(self, task):
         return self.addTask(task)
         
-    def add_lambda(self, fcn, *params, timeout = None, after=None, promise_data=None, **kwargs):
-        task = FunctionTask(fcn, *params, **kwargs)
-        return self.addTask(task, timeout=timeout, after=after, promise_data=promise_data)
-        
-    @synchronized
-    def insertTask(self, task, timeout = None, after=None, promise_data=None):
-        task.After = task.After or after
+    def insertTask(self, task, timeout = None, promise_data=None):
         self.Queue.insert(task, timeout = timeout)
         task.enqueue()
         task._Promise = Promise(data=promise_data)
@@ -152,10 +143,6 @@ class TaskQueue(Primitive):
         return task._Promise
         
     insert = insertTask
-
-    def insert_lambda(self, fcn, *params, timeout = None, after=None, **kwargs):
-        task = FunctionTask(fcn, *params, **kwargs)
-        return self.insertTask(task, timeout=timeout, after=after, promise_data=promise_data)
         
     def __rshift__(self, task):
         return self.insertTask(task)
@@ -165,34 +152,6 @@ class TaskQueue(Primitive):
         self.StartTimer = None
         self.startThreads()
         
-    @synchronized
-    def arm_timer(self, t):
-        if self.StartTimer is not None:
-            self.StartTimer.cancel()
-        delta = max(0.0, t - time.time())
-        self.StartTimer = Timer(delta, self.timer_fired)
-        self.StartTimer.daemon = True
-        self.StartTimer.start()
-
-    @synchronized
-    def next_wakeup(self):
-        """
-        Returns time to start next task
-        - returns now() if next task is ready to start
-        - None if the queue is empty
-        """
-        tmin = None
-        now = time.time()
-        tasks = self.Queue.items()
-        for task in tasks:
-            t = task.After or now
-            tmin = min(tmin or t, t)
-
-        if tmin is not None and self.Stagger is not None:
-            tmin = max(tmin, self.LastStart + self.Stagger)
-
-        return tmin
-
     @synchronized
     def startThreads(self):
         #print "startThreads() entry"
@@ -200,31 +159,25 @@ class TaskQueue(Primitive):
             while self.Queue \
                     and (self.NWorkers is None or len(self.Threads) < self.NWorkers) \
                     and not self.Held:
-
-                if self.Stagger and time.time() < self.LastStart + self.Stagger:
-                    break
-
-                # find next task ready to start (time > task.After)
-                for task in self.Queue.items():
-                    if task.After is None or task.After < time.time():
-                        self.Queue.remove(task)
+                if self.Stagger > 0.0:
+                    delta = self.LastStart + self.Stagger - time.time()
+                    #print "arming timer..."
+                    if delta > 0.0:
+                        if self.StartTimer is None:
+                            self.StartTimer = Timer(delta, self.timer_fired)
+                            self.StartTimer.daemon = True
+                            self.StartTimer.start()
                         break
-                else:
-                    task = None
-                
-                if task is not None:
-                    self.LastStart = time.time()
-                    t = self.ExecutorThread(self, task)
-                    t.kind = "%s.task" % (self.kind,)
-                    self.Threads.append(t)
-                    self.call_delegate("taskIsStarting", self, task)
-                    t.start()
-                    self.call_delegate("taskStarted", self, task)
-                    
-            tnext = self.next_wakeup()
-            if tnext is not None:
-                self.arm_timer(tnext)
-
+                self.LastStart = time.time()
+                task = self.Queue.pop()
+                t = self.ExecutorThread(self, task)
+                t.kind = "%s.task" % (self.kind,)
+                self.Threads.append(t)
+                self.call_delegate("taskIsStarting", self, task)
+                t.start()
+                self.call_delegate("taskStarted", self, task)
+                #print("thread started")
+            
     @synchronized
     def threadEnded(self, t):
         #print("queue.threadEnded: ", t)
