@@ -11,7 +11,7 @@ class T(Task):
 		self.Id = id
 
 	def run(self):
-	    print "task %d: t=%s" % (self.Id, time.ctime(time.time()))
+	    print ("task %d: t=%s" % (self.Id, time.ctime(time.time())))
 
 tq = TaskQueue(1, capacity=10, stagger=1.0)
 
@@ -22,9 +22,9 @@ def printStats():
 
 for i in range(100):
 	t = T(i)
-	print "submitting task %d..." % (i,)
+	print ("submitting task %d..." % (i,))
 	tq << t
-	print "task %d submitted" % (i,)
+	print ("task %d submitted" % (i,))
 	time.sleep(0.2+random.random())
 	if i and i % 10 == 0:
 		Timer(0.5, printStats).start()
